<?php

namespace Island\Island\Http\Controllers;

use App\Http\Controllers\ResourceController as BaseController;
use Form;
use Island\Island\Http\Requests\IslandsRequest;
use Island\Island\Interfaces\IslandsRepositoryInterface;
use Island\Island\Models\Islands;

/**
 * Resource controller class for islands.
 */
class IslandsResourceController extends BaseController
{

    /**
     * Initialize islands resource controller.
     *
     * @param type IslandsRepositoryInterface $islands
     *
     * @return null
     */
    public function __construct(IslandsRepositoryInterface $islands)
    {
        parent::__construct();
        $this->repository = $islands;
        $this->repository
            ->pushCriteria(\Litepie\Repository\Criteria\RequestCriteria::class)
            ->pushCriteria(\Island\Island\Repositories\Criteria\IslandsResourceCriteria::class);
    }

    /**
     * Display a list of islands.
     *
     * @return Response
     */
    public function index(IslandsRequest $request)
    {
        $view = $this->response->theme->listView();

        if ($this->response->typeIs('json')) {
            $function = camel_case('get-' . $view);
            return $this->repository
                ->setPresenter(\Island\Island\Repositories\Presenter\IslandsPresenter::class)
                ->$function();
        }

        $islands = $this->repository->paginate();

        return $this->response->setMetaTitle(trans('island::islands.names'))
            ->view('island::islands.index', true)
            ->data(compact('islands', 'view'))
            ->output();
    }

    /**
     * Display islands.
     *
     * @param Request $request
     * @param Model   $islands
     *
     * @return Response
     */
    public function show(IslandsRequest $request, Islands $islands)
    {

        if ($islands->exists) {
            $view = 'island::islands.show';
        } else {
            $view = 'island::islands.new';
        }

        return $this->response->setMetaTitle(trans('app.view') . ' ' . trans('island::islands.name'))
            ->data(compact('islands'))
            ->view($view, true)
            ->output();
    }

    /**
     * Show the form for creating a new islands.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(IslandsRequest $request)
    {

        $islands = $this->repository->newInstance([]);
        return $this->response->setMetaTitle(trans('app.new') . ' ' . trans('island::islands.name')) 
            ->view('island::islands.create', true) 
            ->data(compact('islands'))
            ->output();
    }

    /**
     * Create new islands.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(IslandsRequest $request)
    {
        try {
            $attributes              = $request->all();
            $attributes['user_id']   = user_id();
            $attributes['user_type'] = user_type();
            $islands                 = $this->repository->create($attributes);

            return $this->response->message(trans('messages.success.created', ['Module' => trans('island::islands.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('island/islands/' . $islands->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/island/islands'))
                ->redirect();
        }

    }

    /**
     * Show islands for editing.
     *
     * @param Request $request
     * @param Model   $islands
     *
     * @return Response
     */
    public function edit(IslandsRequest $request, Islands $islands)
    {
        return $this->response->setMetaTitle(trans('app.edit') . ' ' . trans('island::islands.name'))
            ->view('island::islands.edit', true)
            ->data(compact('islands'))
            ->output();
    }

    /**
     * Update the islands.
     *
     * @param Request $request
     * @param Model   $islands
     *
     * @return Response
     */
    public function update(IslandsRequest $request, Islands $islands)
    {
        try {
            $attributes = $request->all();

            $islands->update($attributes);
            return $this->response->message(trans('messages.success.updated', ['Module' => trans('island::islands.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('island/islands/' . $islands->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('island/islands/' . $islands->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the islands.
     *
     * @param Model   $islands
     *
     * @return Response
     */
    public function destroy(IslandsRequest $request, Islands $islands)
    {
        try {

            $islands->delete();
            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('island::islands.name')]))
                ->code(202)
                ->status('success')
                ->url(guard_url('island/islands/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('island/islands/' . $islands->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove multiple islands.
     *
     * @param Model   $islands
     *
     * @return Response
     */
    public function delete(IslandsRequest $request, $type)
    {
        try {
            $ids = hashids_decode($request->input('ids'));

            if ($type == 'purge') {
                $this->repository->purge($ids);
            } else {
                $this->repository->delete($ids);
            }

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('island::islands.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('island/islands'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/island/islands'))
                ->redirect();
        }

    }

    /**
     * Restore deleted islands.
     *
     * @param Model   $islands
     *
     * @return Response
     */
    public function restore(IslandsRequest $request)
    {
        try {
            $ids = hashids_decode($request->input('ids'));
            $this->repository->restore($ids);

            return $this->response->message(trans('messages.success.restore', ['Module' => trans('island::islands.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('/island/islands'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/island/islands/'))
                ->redirect();
        }

    }

}
